Package: neo2R
Type: Package
Title: Neo4j to R
Version: 3.0.0
Authors@R: c(
    person(
        "Patrice", "Godard",
        role = c("aut", "cre","cph"),
        email = "patrice.godard@gmail.com",
        comment = c(ORCID = "0000-0001-6257-9730")
    ),
    person(
        "Eusebiu", "Marcu",
        role = "ctb",
        email = "marcueusebiu@gmail.com"
    )
    )
Description: The aim of neo2R is to provide simple and low level connectors
   for querying neo4j graph databases (<https://neo4j.com/>).
   The objects returned by the query functions are either lists or data.frames
   with very little post-processing.
   It allows fast processing of queries returning many records.
   And it let the users handle post-processing according to the data model
   and their needs.
URL: https://github.com/patzaw/neo2r
BugReports: https://github.com/patzaw/neo2r/issues
Depends: R (>= 4.1)
Imports: jsonlite, httr2, utils
SystemRequirements: neo4j (>=3) <https://neo4j.com/>
License: GPL-3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 07:11:44 UTC; pgodard
Author: Patrice Godard [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-6257-9730>),
  Eusebiu Marcu [ctb]
Maintainer: Patrice Godard <patrice.godard@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-18 08:20:02 UTC
Built: R 4.6.0; ; 2026-05-18 10:04:31 UTC; unix

Package: neo2R
Type: Package
Title: Neo4j to R
Version: 3.0.0
Authors@R: c(
    person(
        "Patrice", "Godard",
        role = c("aut", "cre","cph"),
        email = "patrice.godard@gmail.com",
        comment = c(ORCID = "0000-0001-6257-9730")
    ),
    person(
        "Eusebiu", "Marcu",
        role = "ctb",
        email = "marcueusebiu@gmail.com"
    )
    )
Description: The aim of neo2R is to provide simple and low level connectors
   for querying neo4j graph databases (<https://neo4j.com/>).
   The objects returned by the query functions are either lists or data.frames
   with very little post-processing.
   It allows fast processing of queries returning many records.
   And it let the users handle post-processing according to the data model
   and their needs.
URL: https://github.com/patzaw/neo2r
BugReports: https://github.com/patzaw/neo2r/issues
Depends: R (>= 4.1)
Imports: jsonlite, httr2, utils
SystemRequirements: neo4j (>=3) <https://neo4j.com/>
License: GPL-3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 07:11:44 UTC; pgodard
Author: Patrice Godard [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-6257-9730>),
  Eusebiu Marcu [ctb]
Maintainer: Patrice Godard <patrice.godard@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-18 08:20:02 UTC
Built: R 4.5.2; ; 2026-05-18 08:46:29 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cypher)
export(graphRequest)
export(import_from_df)
export(multicypher)
export(prepCql)
export(readCql)
export(startGraph)
import(httr2)
import(jsonlite)
import(utils)

#######################################################

#' Prepares a CQL query from a character vector
#'
#' @param ... character vectors with cQL commands
#'
#' @return A well formated CQL query
#'
#' @seealso [cypher()] and [readCql()]
#'
#' @examples prepCql(c(
#'  "MATCH (n)",
#'  "RETURN n"
#' ))
#'
#' @export
#'
prepCql <- function(...) {
  cql <- paste(..., collapse = " ")
  return(paste(sub(";[[:blank:]]*$", "", cql)))
}


#######################################################

#' Parse a CQL file and returned the prepared queries
#'
#' @param file the name of the file to be parsed
#'
#' @return A character vector of well formated CQL queries
#'
#' @seealso [cypher()] and [prepCql()]
#'
#' @export
#'
readCql <- function(file) {
  rq <- readLines(file)
  rq <- sub("//.*", "", rq)
  rq <- rq[which(rq != "")]
  qend <- grep(";$", rq)
  qstart <- c(1, (qend[1:(length(qend))] + 1)[-length(qend)])
  toRet <- apply(
    cbind(qstart, qend),
    1,
    function(x) {
      prepCql(rq[x[1]:x[2]])
    }
  )
  return(toRet)
}

#' Prepare connection to neo4j database
#'
#' @param url the DB url
#' @param database the name of the database. If NA (default) it will use "data"
#' with versions 3.. of Neo4j and "neo4j" with versions 4..
#' @param username the neo4j user name
#' (default: NA; works only if authentication has been disabled in neo4j by
#' setting NEO4J.AUTH=none)
#' @param password the neo4j user password
#' (default: NA; works only if authentication has been disabled in neo4j by
#' setting NEO4J.AUTH=none)
#' @param importPath path to the import directory
#' (default: NA => no import directory). Import only works with local neo4j
#' instance.
#' @param .opts a named list identifying the curl
#' options for the handle (see [httr2::req_options()] and curl option names
#' for a complete list of available options;
#' for example: `.opts = list(ssl_verifypeer = 0)`). Moreover, this parameter
#' can be used to pass additional headers to the graph requests as
#' "extendedHeaders": it is useful, for example, for OAuth access
#' delegation (see details).
#' @param check check the connection before returning it (default: TRUE).
#' Set to false when connection to the "system" database
#' @param api the HTTP API to use: `"tx"` for the legacy Transactional Cypher
#' HTTP API (default for self-managed instances), `"v2"` for the Neo4j Query
#' API v2 (required for Aura, available on self-managed Neo4j >= 5.19), or
#' `"auto"` (default) to detect automatically — Aura URLs
#' (*.databases.neo4j.io) select `"v2"`, all others select `"tx"`.
#'
#'
#' @details The "ssl.verifypeer" logical option available in the RCurl package
#' used in former versions of neo2R (<= 2.2.0) is
#' not recognized by [httr2::req_options()].
#' However, for backward compatibility, if it is used, it is translated into
#' the "ssl_verifypeer" integer curl option with a warning message.
#'
#' Headers in `.opts$extendedHeaders` are added to, or overwrite,
#' the default Neo4j headers.
#' If there is a `.opts$extendedHeaders[["Authorization"]]` value, the
#' default Neo4j "Authorization" header (user credentials) is provided
#' automaticaly as "X-Authorization". This mechanism is used for OAuth access
#' delegation.
#'
#' @return A connection to the graph DB:
#' a list with the url and necessary headers
#'
#' @export
#'
startGraph <- function(
  url,
  database = NA,
  username = NA,
  password = NA,
  importPath = NA,
  .opts = list(),
  check = TRUE,
  api = c("auto", "tx", "v2")
) {
  api <- match.arg(api)

  if ("ssl.verifypeer" %in% names(.opts)) {
    .opts$ssl_verifypeer <- as.integer(.opts$ssl.verifypeer)
    .opts$ssl.verifypeer <- NULL
    warning(
      "'ssl.verifypeer' option has been automatically converted into ",
      "'ssl_verifypeer' integer curl option. ",
      "You should use the 'ssl_verifypeer' integer option ",
      "to avoid this warning."
    )
  }

  ## Process URL and guess protocol ----
  protocol <- grep("^https://", url)
  if (length(protocol) == 1) {
    protocol = "https://"
    url <- sub("^https://", "", url)
  } else {
    protocol <- "http://"
    url <- sub("^http://", "", url)
  }
  url <- paste0(
    protocol,
    sub("[/]db.*$", "", url)
  )
  ## Set general header ----
  neo4jHeaders <- list(
    'Accept' = 'application/json; charset=UTF-8;',
    'Content-Type' = 'application/json',
    'X-Stream' = TRUE,
    'Authorization' = paste(
      "Basic",
      jsonlite::base64_enc(paste(username, password, sep = ":"))
    )
  )

  ## Append other headers - swap Authorization headers for OAuth ----
  extendedHeaders <- .opts$extendedHeaders
  .opts$extendedHeaders <- NULL
  if (!is.null(extendedHeaders[["Authorization"]])) {
    temp_auth <- neo4jHeaders[["Authorization"]]
    neo4jHeaders[["Authorization"]] <- extendedHeaders[["Authorization"]]
    neo4jHeaders[["X-Authorization"]] <- temp_auth
    extendedHeaders <- extendedHeaders[setdiff(
      names(extendedHeaders),
      "Authorization"
    )]
  }
  overw <- intersect(names(extendedHeaders), names(neo4jHeaders))
  if (length(overw) > 0) {
    warning(
      "The following default headers are overwritten by user values: ",
      paste(overw, sep = ", ")
    )
    neo4jHeaders <- neo4jHeaders[setdiff(
      names(neo4jHeaders),
      names(extendedHeaders)
    )]
  }
  neo4jHeaders <- c(neo4jHeaders, extendedHeaders)

  toRet <- list(
    url = url,
    headers = neo4jHeaders,
    importPath = importPath,
    .opts = .opts
  )
  ## Find neo4j version and database name ----
  conStatus <- graphRequest(toRet, "", "GET", "")
  if (
    is.na(conStatus$header["status"]) || conStatus$header["status"] != "200"
  ) {
    print(conStatus$header)
    stop("Cannot connect to the Neo4j database")
  }
  if ("neo4j_version" %in% names(conStatus$result)) {
    toRet$version = unlist(strsplit(
      conStatus$result$neo4j_version,
      split = "[.]"
    ))
  } else {
    if (is.na(database)) {
      database <- "data"
    }
    conStatus <- graphRequest(toRet, sprintf("/db/%s/", database), "GET", "")
    if (
      is.na(conStatus$header["status"]) || conStatus$header["status"] != "200"
    ) {
      print(conStatus$header)
      stop("Cannot connect to the Neo4j database")
    }
    if ("neo4j_version" %in% names(conStatus$result)) {
      toRet$version = unlist(strsplit(
        conStatus$result$neo4j_version,
        split = "[.]"
      ))
    } else {
      stop("Unknown version of neo4j")
    }
  }
  .v1 <- suppressWarnings(as.integer(toRet$version[1]))
  .year_based <- !is.na(.v1) && .v1 >= 2025
  if (!toRet$version[1] %in% c("3", "4", "5") && !.year_based) {
    print(toRet["version"])
    stop(
      "Only Neo4j versions 3, 4, 5 and year-based versions ",
      "(>= 2025) are supported"
    )
  }
  ## Define database name ----
  if (toRet$version[1] == "3") {
    toRet$database <- ifelse(is.na(database), "data", database)
  }
  if (toRet$version[1] %in% c("4", "5") || .year_based) {
    toRet$database <- ifelse(is.na(database), "neo4j", database)
  }
  ## Determine API ----
  if (api == "auto") {
    api <- if (
      grepl("\\.databases\\.neo4j\\.io", toRet$url, ignore.case = TRUE) ||
        .year_based
    ) {
      "v2"
    } else {
      "tx"
    }
  }
  toRet$api <- api
  ## Define cypher endpoint ----
  if (api == "v2") {
    toRet$cypher_endpoint <- sprintf("/db/%s/query/v2", toRet$database)
  } else {
    if (toRet$version[1] == "3") {
      toRet$cypher_endpoint <- sprintf(
        "/db/%s/transaction/commit",
        toRet$database
      )
    }
    if (toRet$version[1] %in% c("4", "5") || .year_based) {
      toRet$cypher_endpoint <- sprintf("/db/%s/tx/commit", toRet$database)
    }
  }
  ## Final connection check ----
  if (check) {
    cypher(toRet, "match (n) return n limit 1", result = "graph")
  }
  return(toRet)
}

#' Run a curl request on a neo4j graph
#'
#' @param graph the neo4j connection
#' @param endpoint the endpoint for the request. To list all the available
#' endpoints:
#' `graphRequest(graph, endpoint="", customrequest="GET", postText="")$result`
#' @param customrequest the type of request: "POST" (default) or "GET"
#' @param postText the request body
#'
#' @return A list with the "header" and the "result" of the request (invisible)
#'
#' @seealso [startGraph()] and [cypher()]
#'
#' @export
#'
graphRequest <- function(
  graph,
  endpoint, #"transaction/commit"
  customrequest = c("POST", "GET"),
  postText
) {
  customrequest <- match.arg(customrequest)
  postfields <- jsonlite::toJSON(postText, auto_unbox = T)

  req <- httr2::request(paste0(graph$url, endpoint)) |>
    httr2::req_method(customrequest)
  req <- do.call(httr2::req_headers, c(list(req), graph$headers))
  if (length(graph$.opts) > 0) {
    req <- do.call(httr2::req_options, c(list(req), graph$.opts))
  }
  if (customrequest == "POST") {
    req <- httr2::req_body_raw(req, charToRaw(postfields))
  }
  resp <- req |>
    httr2::req_error(is_error = \(r) FALSE) |>
    httr2::req_perform()
  invisible(list(
    header = c(status = httr2::resp_status(resp)),
    result = jsonlite::fromJSON(
      httr2::resp_body_string(resp),
      simplifyVector = FALSE
    )
  ))
}

###############################################################################@
#' Run a cypher query
#'
#' @param graph the neo4j connection
#' @param query the cypher query
#' @param parameters parameters for the cypher query.
#' @param result the way to return results. "row" will return a data frame
#' and "graph" will return a list of nodes, a list of relationships
#' and a list of paths (vectors of relationships identifiers).
#' @param arraysAsStrings if result="row" and arraysAsStrings is TRUE (default)
#' array from neo4j are converted to strings and array elements are
#' separated by eltSep.
#' @param eltSep if result="row" and arraysAsStrings is TRUE (default)
#' array from neo4j are converted to strings and array elementes are
#' separated by eltSep.
#'
#' @return The "result" of the query (invisible). See the "result" param.
#'
#' @seealso [multicypher()], [startGraph()], [prepCql()],
#' [readCql()] and [graphRequest()]
#'
#' @examples \dontrun{
#' # 2 identical queries
#' result <- cypher(
#'    graph=graph,
#'    query='match (n {value:$value}) return n',
#'    parameters=list(value="100"),
#'    result="graph"
#' )
#' result <- cypher(
#'    graph=graph,
#'    query='match (n {value:"100"}) return n',
#'    result="graph"
#' )
#' }
#'
#' @export
#'
cypher <- function(
  graph,
  query,
  parameters = NULL,
  result = c("row", "graph"),
  arraysAsStrings = TRUE,
  eltSep = " || "
) {
  result = match.arg(result)
  endpoint <- graph$cypher_endpoint
  api <- if (is.null(graph$api)) "tx" else graph$api

  if (api == "v2") {
    postText <- list(statement = query)
    if (!is.null(parameters)) postText$parameters <- parameters
  } else {
    postText <- list(
      statements = list(list(
        statement = query,
        resultDataContents = list(result)
      ))
    )
    if (!is.null(parameters)) {
      postText$statements[[1]]$parameters <- parameters
    }
  }

  raw_result <- graphRequest(
    graph = graph,
    endpoint = endpoint,
    customrequest = "POST",
    postText = postText
  )$result
  errors <- raw_result$errors
  if (length(errors) > 0) {
    devnull <- lapply(errors, lapply, message)
    stop("neo4j error")
  }
  if (api == "v2") {
    if (result == "row") {
      toRet <- process_row_v2(
        raw_result$data,
        arraysAsStrings = arraysAsStrings,
        eltSep = eltSep
      )
    } else {
      toRet <- process_graph_v2(raw_result$data)
    }
  } else {
    if (result == "row") {
      toRet <- process_row(
        raw_result$results[[1]],
        arraysAsStrings = arraysAsStrings,
        eltSep = eltSep
      )
    }
    if (result == "graph") {
      toRet <- process_graph(raw_result$results[[1]]$data)
    }
  }
  invisible(toRet)
}

###############################################################################@
#' Run a multiple cypher queriers
#'
#' @param graph the neo4j connection
#' @param queries queries to submit. It can be either a character vector
#' for which each element corresponds to a cypher query. Or it can be
#' a list of lists with the following slots:
#' - **query** (mandatory): A single character corresponding to the cypher
#' query.
#' - **parameters** (optional): A set of parameters specific for
#' this query. If not provided, the *parameters* parameter of the function is
#' used  (see below).
#' - **result** (optional): The specific way to return the results of
#' this query. If not provided, the *result* parameter of the function is
#' used  (see below).
#' @param parameters default parameters for the cypher queries.
#' @param result default way to return results. "row" will return a data frame
#' and "graph" will return a list of nodes, a list of relationships
#' and a list of paths (vectors of relationships identifiers).
#' @param arraysAsStrings if result="row" and arraysAsStrings is TRUE (default)
#' array from neo4j are converted to strings and array elements are
#' separated by eltSep.
#' @param eltSep if result="row" and arraysAsStrings is TRUE (default)
#' array from neo4j are converted to strings and array elementes are
#' separated by eltSep.
#'
#' @return A list of "result" of the queries (invisible).
#' See the "result" param.
#'
#' @seealso [cypher()], [startGraph()], [prepCql()],
#' [readCql()] and [graphRequest()]
#'
#' @examples \dontrun{
#' result <- multicypher(
#'    graph,
#'    queries=list(
#'       q1="match (n) return n.value limit 5",
#'       q2=list(
#'          query="match (f {value:$val})-[r]->(t) return f, r, t limit 5",
#'          result="graph",
#'          parameters=list(val=100)
#'       )
#'    )
#' )
#' }
#'
#' @export
#'
multicypher <- function(
  graph,
  queries,
  parameters = NULL,
  result = c("row", "graph"),
  arraysAsStrings = TRUE,
  eltSep = " || "
) {
  result = match.arg(result)
  endpoint <- graph$cypher_endpoint
  api <- if (is.null(graph$api)) "tx" else graph$api
  qnames <- names(queries)

  ## Query API v2: one request per query ----
  if (api == "v2") {
    toRet <- lapply(
      unname(queries),
      function(query) {
        if (is.character(query)) {
          cypher(
            graph,
            query = query,
            parameters = parameters,
            result = result,
            arraysAsStrings = arraysAsStrings,
            eltSep = eltSep
          )
        } else {
          if (!"query" %in% names(query)) {
            stop('Parameterized query should have a "query" slot')
          }
          qparams <- if (!is.null(query$parameters)) {
            query$parameters
          } else {
            parameters
          }
          qresult <- if (!is.null(query$result)) {
            match.arg(query$result, c("row", "graph"))
          } else {
            result
          }
          cypher(
            graph,
            query = query$query,
            parameters = qparams,
            result = qresult,
            arraysAsStrings = arraysAsStrings,
            eltSep = eltSep
          )
        }
      }
    )
    names(toRet) <- qnames
    return(invisible(toRet))
  }

  statements <- lapply(
    unname(queries),
    function(query) {
      if (!is.character(query) && !is.list(query)) {
        stop("Each query should be character or a list")
      }
      if (is.character(query)) {
        toRet <- list(statement = query)
      }
      if (is.list(query)) {
        if (!"query" %in% names(query)) {
          stop('Parameterized query should have a "query" slot')
        }
        if (!is.character(query$query)) {
          stop('The "query" slot should be a character')
        }
        toRet <- list(statement = query$query)
        if ("result" %in% names(query)) {
          if (!query$result %in% c("row", "graph")) {
            stop('The "result" slot should be "row" or "graph"')
          }
          toRet$resultDataContents <- list(query$result)
        }
        if (!is.null(query$parameters)) {
          toRet$parameters <- query$parameters
        }
      }
      if (is.null(toRet$resultDataContents)) {
        toRet$resultDataContents <- list(result)
      }
      if (is.null(toRet$parameters) && !is.null(parameters)) {
        toRet$parameters <- parameters
      }
      return(toRet)
    }
  )
  postText <- list(statements = statements)

  results <- graphRequest(
    graph = graph,
    endpoint = endpoint,
    customrequest = "POST",
    postText = postText
  )$result
  errors <- results$errors
  if (length(errors) > 0) {
    devnull <- lapply(errors, lapply, message)
    stop("neo4j error")
  }

  toRet <- lapply(
    1:length(results$results),
    function(i) {
      results <- results$results[[i]]
      if (statements[[i]]$resultDataContents == "row") {
        return(process_row(
          results,
          arraysAsStrings = arraysAsStrings,
          eltSep = eltSep
        ))
      }
      if (statements[[i]]$resultDataContents == "graph") {
        return(process_graph(results$d))
      }
    }
  )
  names(toRet) <- qnames
  invisible(toRet)
}


###############################################################################@
## Helpers ----
process_row <- function(results, arraysAsStrings, eltSep) {
  if (length(results$data) == 0) {
    toRet <- NULL
  } else {
    if (!is.null(names(results$data[[1]][[1]]))) {
      warning(
        "Complex data from query ==> you should shift to 'graph' result."
      )
    }
    columns <- do.call(c, results$columns)
    toRet <- do.call(rbind, lapply(results$data, function(x) x$row))
    toRet[sapply(toRet, is.null)] <- NA
    toRet <- data.frame(toRet, stringsAsFactors = FALSE)
    if (all(sapply(toRet, class) == "list")) {
      for (i in 1:ncol(toRet)) {
        if (max(unlist(sapply(toRet[[i]], length))) == 1) {
          toRet[, i] <- unlist(toRet[, i])
        } else {
          if (arraysAsStrings) {
            toRet[, i] <- unlist(lapply(
              toRet[, i],
              paste,
              collapse = eltSep
            ))
          }
        }
      }
    }
    colnames(toRet) <- columns
  }
  return(toRet)
}

process_row_v2 <- function(data, arraysAsStrings, eltSep) {
  if (is.null(data) || length(data$values) == 0) {
    return(NULL)
  }
  process_row(
    list(
      columns = data$fields,
      data = lapply(data$values, function(row) list(row = row))
    ),
    arraysAsStrings,
    eltSep
  )
}

process_graph <- function(d) {
  if (is.null(d) || length(d) == 0) {
    return(NULL)
  }
  nodes <- unique(do.call(c, lapply(d, function(x) x$graph$nodes)))
  names(nodes) <- unlist(lapply(nodes, function(n) n$id))
  relationships <- unique(do.call(
    c,
    lapply(
      d,
      function(x) x$graph$relationships
    )
  ))
  names(relationships) <- unlist(lapply(relationships, function(n) n$id))
  p <- lapply(
    d,
    function(x) {
      unique(unlist(lapply(x$graph$relationships, function(y) y$id)))
    }
  )
  p <- p[which(!unlist(lapply(p, is.null)))]
  toRet <- list(nodes = nodes, relationships = relationships, paths = p)
}

process_graph_v2 <- function(d) {
  if (is.null(d) || length(d$values) == 0) {
    return(NULL)
  }
  is_node <- function(v) {
    is.list(v) && !is.null(v$elementId) && !is.null(v$labels) && is.null(v$type)
  }
  is_rel <- function(v) {
    is.list(v) &&
      !is.null(v$elementId) &&
      !is.null(v$type) &&
      !is.null(v$startNodeElementId)
  }
  nodes <- list()
  relationships <- list()
  p <- list()
  for (row in d$values) {
    row_rel_ids <- character(0)
    for (val in row) {
      if (is_node(val)) {
        nodes[[val$elementId]] <- val
      } else if (is_rel(val)) {
        relationships[[val$elementId]] <- val
        row_rel_ids <- c(row_rel_ids, val$elementId)
      }
    }
    if (length(row_rel_ids) > 0) p[[length(p) + 1]] <- row_rel_ids
  }
  list(nodes = nodes, relationships = relationships, paths = p)
}

#' Imports a data.frame in the neo4j graph database
#'
#' This function only works with localhost Neo4j instances.
#'
#' @param graph the neo4j connection
#' @param cql the CQL query to be applied on each row of toImport.
#' Use the 'row' prefix to refer to the data.frame column.
#' @param toImport the data.frame to be imported as "row".
#' Use "row.FIELD" in the cql query to refer to one FIELD of the toImport
#' data.frame
#' @param periodicCommit use periodic commit when loading the data
#' (default: 10000).
#' @param by number of rows to send by batch (default: Inf).
#' Can be an alternative to periodic commit.
#' @param ... further parameters for [cypher()]
#'
#' @seealso [cypher()]
#'
#' @export
#'
import_from_df <- function(
  graph,
  cql,
  toImport,
  periodicCommit = 1000,
  by = Inf,
  ...
) {
  stopifnot(
    is.data.frame(toImport),
    length(by) == 1,
    is.numeric(by),
    by > 0,
    length(periodicCommit) == 1,
    is.na(periodicCommit) ||
      (is.numeric(periodicCommit) && periodicCommit > 0)
  )
  importPath <- graph$importPath
  stopifnot(
    !is.null(importPath),
    !is.na(importPath)
  )
  if (!file.exists(importPath)) {
    stop(sprintf("Import path (%s) does not exist.", importPath))
  }
  tf <- tempfile(tmpdir = importPath)
  for (cn in colnames(toImport)) {
    toImport[, cn] <- as.character(toImport[, cn, drop = TRUE])
  }
  # pc <- c()
  # if(is.numeric(periodicCommit) && length(periodicCommit)==1){
  #    if(graph$version[[1]]!=5){
  #       pc <- sprintf("USING PERIODIC COMMIT %s", periodicCommit)
  #    }else{
  #       warning(
  #          "Periodic commit not supported for Neo4j >= 5.\n",
  #          "Consider the 'by' parameter."
  #       )
  #    }
  # }
  # cql <- prepCql(c(
  #    pc,
  #    paste0(
  #       'LOAD CSV WITH HEADERS FROM "file:',
  #       ifelse(
  #          !is.null(importPath),
  #          file.path("", basename(tf)),
  #          tf
  #       ),
  #       '" AS row '# FIELDTERMINATOR "\\t"'
  #    ),
  #    cql
  # ))
  load_string <- paste0(
    'LOAD CSV WITH HEADERS FROM "file:',
    ifelse(
      !is.null(importPath),
      file.path("", basename(tf)),
      tf
    ),
    '" AS row ' # FIELDTERMINATOR "\\t"'
  )
  if (!is.na(periodicCommit)) {
    if (graph$version[[1]] != 5) {
      cql <- prepCql(c(
        sprintf("USING PERIODIC COMMIT %s", periodicCommit),
        load_string,
        cql
      ))
    } else {
      cql <- prepCql(c(
        load_string,
        "CALL {WITH row",
        cql,
        "}",
        sprintf("IN TRANSACTIONS OF %s ROWS RETURN count(*)", periodicCommit)
      ))
    }
  } else {
    cql <- prepCql(c(
      load_string,
      cql
    ))
  }
  if (nrow(toImport) <= 1000) {
    taken <- 0
    while (taken < nrow(toImport)) {
      totake <- min(taken + by, nrow(toImport))
      utils::write.table(
        toImport[(taken + 1):totake, , drop = FALSE],
        file = tf,
        sep = ",", #"\t",
        quote = T,
        na = '',
        row.names = F,
        col.names = T
      )
      on.exit(file.remove(tf))
      toRet <- cypher(graph = graph, query = cql, ...)
      taken <- totake
    }
    invisible(toRet)
  } else {
    utils::write.table(
      toImport[c(1:1000), , drop = FALSE],
      file = tf,
      sep = ",", #"\t",
      quote = T,
      na = '',
      row.names = F,
      col.names = T
    )
    on.exit(file.remove(tf))
    toRet <- cypher(graph = graph, query = cql, ...)
    cypher(graph = graph, query = 'CALL db.resampleOutdatedIndexes();')
    taken <- 1000
    while (taken < nrow(toImport)) {
      totake <- min(taken + by, nrow(toImport))
      utils::write.table(
        toImport[(taken + 1):totake, , drop = FALSE],
        file = tf,
        sep = ",", #"\t",
        quote = T,
        na = '',
        row.names = F,
        col.names = T
      )
      on.exit(file.remove(tf))
      toRet <- cypher(graph = graph, query = cql, ...)
      taken <- totake
    }
    invisible(toRet)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_from_df.R
\name{import_from_df}
\alias{import_from_df}
\title{Imports a data.frame in the neo4j graph database}
\usage{
import_from_df(graph, cql, toImport, periodicCommit = 1000, by = Inf, ...)
}
\arguments{
\item{graph}{the neo4j connection}

\item{cql}{the CQL query to be applied on each row of toImport.
Use the 'row' prefix to refer to the data.frame column.}

\item{toImport}{the data.frame to be imported as "row".
Use "row.FIELD" in the cql query to refer to one FIELD of the toImport
data.frame}

\item{periodicCommit}{use periodic commit when loading the data
(default: 10000).}

\item{by}{number of rows to send by batch (default: Inf).
Can be an alternative to periodic commit.}

\item{...}{further parameters for \code{\link[=cypher]{cypher()}}}
}
\description{
This function only works with localhost Neo4j instances.
}
\seealso{
\code{\link[=cypher]{cypher()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphRequest.R
\name{graphRequest}
\alias{graphRequest}
\title{Run a curl request on a neo4j graph}
\usage{
graphRequest(graph, endpoint, customrequest = c("POST", "GET"), postText)
}
\arguments{
\item{graph}{the neo4j connection}

\item{endpoint}{the endpoint for the request. To list all the available
endpoints:
\code{graphRequest(graph, endpoint="", customrequest="GET", postText="")$result}}

\item{customrequest}{the type of request: "POST" (default) or "GET"}

\item{postText}{the request body}
}
\value{
A list with the "header" and the "result" of the request (invisible)
}
\description{
Run a curl request on a neo4j graph
}
\seealso{
\code{\link[=startGraph]{startGraph()}} and \code{\link[=cypher]{cypher()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cypher.R
\name{multicypher}
\alias{multicypher}
\title{Run a multiple cypher queriers}
\usage{
multicypher(
  graph,
  queries,
  parameters = NULL,
  result = c("row", "graph"),
  arraysAsStrings = TRUE,
  eltSep = " || "
)
}
\arguments{
\item{graph}{the neo4j connection}

\item{queries}{queries to submit. It can be either a character vector
for which each element corresponds to a cypher query. Or it can be
a list of lists with the following slots:
\itemize{
\item \strong{query} (mandatory): A single character corresponding to the cypher
query.
\item \strong{parameters} (optional): A set of parameters specific for
this query. If not provided, the \emph{parameters} parameter of the function is
used  (see below).
\item \strong{result} (optional): The specific way to return the results of
this query. If not provided, the \emph{result} parameter of the function is
used  (see below).
}}

\item{parameters}{default parameters for the cypher queries.}

\item{result}{default way to return results. "row" will return a data frame
and "graph" will return a list of nodes, a list of relationships
and a list of paths (vectors of relationships identifiers).}

\item{arraysAsStrings}{if result="row" and arraysAsStrings is TRUE (default)
array from neo4j are converted to strings and array elements are
separated by eltSep.}

\item{eltSep}{if result="row" and arraysAsStrings is TRUE (default)
array from neo4j are converted to strings and array elementes are
separated by eltSep.}
}
\value{
A list of "result" of the queries (invisible).
See the "result" param.
}
\description{
Run a multiple cypher queriers
}
\examples{
\dontrun{
result <- multicypher(
   graph,
   queries=list(
      q1="match (n) return n.value limit 5",
      q2=list(
         query="match (f {value:$val})-[r]->(t) return f, r, t limit 5",
         result="graph",
         parameters=list(val=100)
      )
   )
)
}

}
\seealso{
\code{\link[=cypher]{cypher()}}, \code{\link[=startGraph]{startGraph()}}, \code{\link[=prepCql]{prepCql()}},
\code{\link[=readCql]{readCql()}} and \code{\link[=graphRequest]{graphRequest()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cypher.R
\name{cypher}
\alias{cypher}
\title{Run a cypher query}
\usage{
cypher(
  graph,
  query,
  parameters = NULL,
  result = c("row", "graph"),
  arraysAsStrings = TRUE,
  eltSep = " || "
)
}
\arguments{
\item{graph}{the neo4j connection}

\item{query}{the cypher query}

\item{parameters}{parameters for the cypher query.}

\item{result}{the way to return results. "row" will return a data frame
and "graph" will return a list of nodes, a list of relationships
and a list of paths (vectors of relationships identifiers).}

\item{arraysAsStrings}{if result="row" and arraysAsStrings is TRUE (default)
array from neo4j are converted to strings and array elements are
separated by eltSep.}

\item{eltSep}{if result="row" and arraysAsStrings is TRUE (default)
array from neo4j are converted to strings and array elementes are
separated by eltSep.}
}
\value{
The "result" of the query (invisible). See the "result" param.
}
\description{
Run a cypher query
}
\examples{
\dontrun{
# 2 identical queries
result <- cypher(
   graph=graph,
   query='match (n {value:$value}) return n',
   parameters=list(value="100"),
   result="graph"
)
result <- cypher(
   graph=graph,
   query='match (n {value:"100"}) return n',
   result="graph"
)
}

}
\seealso{
\code{\link[=multicypher]{multicypher()}}, \code{\link[=startGraph]{startGraph()}}, \code{\link[=prepCql]{prepCql()}},
\code{\link[=readCql]{readCql()}} and \code{\link[=graphRequest]{graphRequest()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepCql.R
\name{readCql}
\alias{readCql}
\title{Parse a CQL file and returned the prepared queries}
\usage{
readCql(file)
}
\arguments{
\item{file}{the name of the file to be parsed}
}
\value{
A character vector of well formated CQL queries
}
\description{
Parse a CQL file and returned the prepared queries
}
\seealso{
\code{\link[=cypher]{cypher()}} and \code{\link[=prepCql]{prepCql()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepCql.R
\name{prepCql}
\alias{prepCql}
\title{Prepares a CQL query from a character vector}
\usage{
prepCql(...)
}
\arguments{
\item{...}{character vectors with cQL commands}
}
\value{
A well formated CQL query
}
\description{
Prepares a CQL query from a character vector
}
\examples{
prepCql(c(
 "MATCH (n)",
 "RETURN n"
))

}
\seealso{
\code{\link[=cypher]{cypher()}} and \code{\link[=readCql]{readCql()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startGraph.R
\name{startGraph}
\alias{startGraph}
\title{Prepare connection to neo4j database}
\usage{
startGraph(
  url,
  database = NA,
  username = NA,
  password = NA,
  importPath = NA,
  .opts = list(),
  check = TRUE,
  api = c("auto", "tx", "v2")
)
}
\arguments{
\item{url}{the DB url}

\item{database}{the name of the database. If NA (default) it will use "data"
with versions 3.. of Neo4j and "neo4j" with versions 4..}

\item{username}{the neo4j user name
(default: NA; works only if authentication has been disabled in neo4j by
setting NEO4J.AUTH=none)}

\item{password}{the neo4j user password
(default: NA; works only if authentication has been disabled in neo4j by
setting NEO4J.AUTH=none)}

\item{importPath}{path to the import directory
(default: NA => no import directory). Import only works with local neo4j
instance.}

\item{.opts}{a named list identifying the curl
options for the handle (see \code{\link[httr2:req_options]{httr2::req_options()}} and curl option names
for a complete list of available options;
for example: \code{.opts = list(ssl_verifypeer = 0)}). Moreover, this parameter
can be used to pass additional headers to the graph requests as
"extendedHeaders": it is useful, for example, for OAuth access
delegation (see details).}

\item{check}{check the connection before returning it (default: TRUE).
Set to false when connection to the "system" database}

\item{api}{the HTTP API to use: \code{"tx"} for the legacy Transactional Cypher
HTTP API (default for self-managed instances), \code{"v2"} for the Neo4j Query
API v2 (required for Aura, available on self-managed Neo4j >= 5.19), or
\code{"auto"} (default) to detect automatically — Aura URLs
(*.databases.neo4j.io) select \code{"v2"}, all others select \code{"tx"}.}
}
\value{
A connection to the graph DB:
a list with the url and necessary headers
}
\description{
Prepare connection to neo4j database
}
\details{
The "ssl.verifypeer" logical option available in the RCurl package
used in former versions of neo2R (<= 2.2.0) is
not recognized by \code{\link[httr2:req_options]{httr2::req_options()}}.
However, for backward compatibility, if it is used, it is translated into
the "ssl_verifypeer" integer curl option with a warning message.

Headers in \code{.opts$extendedHeaders} are added to, or overwrite,
the default Neo4j headers.
If there is a \code{.opts$extendedHeaders[["Authorization"]]} value, the
default Neo4j "Authorization" header (user credentials) is provided
automaticaly as "X-Authorization". This mechanism is used for OAuth access
delegation.
}
